import os 
from xml.dom import minidom
from lxml import etree


def dict_insert_collision_avoidance(collision_counts, key, value, dic):
    if(key not in dic):
        dic[key] = value
    elif(key + "-" + str(collision_counts) not in dic):
        dic[key + "-" + str(collision_counts)] = value
    else:
        dict_insert_collision_avoidance(collision_counts + 1, key, value, dic)

def extract(input_folder, all_files):
    data = []
    assumptions = True
    for idx, f in enumerate(all_files):
        if("journal.pcbi" not in f):
            print("Not PCBI, skipping.")
            continue
        print("Current Document", idx + 1, "of",  len(all_files)," - ", (idx + 1)/len(all_files) * 100, "% - ", f, " - ", len(data))
        if(not assumptions):
            break

        p = etree.parse(input_folder + f)
        journal_title = p.findall(".//journal-title")
        pub_date = p.findall(".//pub-date")
        

        accept = { 
            "Year": ["2019", "2020", "2021", "2022"],
            "Journal Title": "Computational Biology",
            "Article Type": "Research Article"
        }
        p.findall(".//subject")

        
        sections = p.xpath(f'.//sec[contains(@sec-type, "{"methods"}")]')
        if sections:
            methods = ' '.join(etree.tostring(sections[0], encoding='utf-8', method='text').decode('utf-8').replace('\n', ' ').split())
        else:
            print("    skipping - Requirement violation, Materials and Methods section missing.")
        
        titles = []
        for elem in journal_title:
            titles.append(elem.text)

        title_match = False
        for title in titles:
            title_match |= accept["Journal Title"] in title
        
        if(not title_match):
            print("    skipping - Requirement violation, Titles not in", accept["Journal Title"], "is", titles, "instead")
            continue

        date = []
        date_type = []
        year = ""
        for elem in pub_date:
            ty = elem.attrib["pub-type"]
            formated_date = []
            for x in elem.itertext():
                if x.strip():
                    formated_date.append(x)
            d = ('/'.join(formated_date))

            date_type.append(ty)
            date.append(d)

            if(ty == "epub"):
                year = d.split("/")[2]
    
        
        if(year not in accept["Year"]):
            print("    skipping - Requirement violation, date is not in", accept["Year"], "is", year, "instead.")
            continue

        subj_group = p.findall(".//article-categories")
        subjects = {}
        for subje in subj_group:
            subj = subje.findall(".//subj-group")
            for s in subj:
                if "subj-group-type" in s.attrib:
                    sub = s.attrib["subj-group-type"]
                    subject = s.findall(".//subject")

                    if(sub != ""):
                        if(sub not in subjects):
                            subjects[sub] = set()
                        for s_ in subject:
                            subjects[sub].add(s_.text)
        
        if "heading" in subjects:
            if list(subjects["heading"])[0] not in accept["Article Type"]:
                print("    skipping - Requirement violation, article type is not in", accept["Article Type"], "is", subjects["heading"], "instead.")
        else:
            print("    skipping - Requirement violation, no subj_group heading, thus no", accept["Article Type"])
            continue

        supplementary_data = p.findall(".//supplementary-material")
        if not supplementary_data:
            print("    skipping - Requirement violation, no supplementary-material found")
            continue
    
        article_meta = p.find(".//article-meta")
        article_title = p.find(".//article-title").text


        name = p.findall(".//meta-name")
        value = p.xpath(".//meta-value")
        meta_value = None
        other = []
        for name_elem, value_elem in zip(name, value):
            if("Data Availability" in name_elem.text):
                meta_value = ''.join(value_elem.itertext())
            else:
                other.append({ "name": name_elem.text, "value": etree.tostring(value_elem).decode() })
        if not meta_value:
            print("    skipping - Requirement violation, no Data Availability found")
            
        journal_meta = p.findall(".//journal-meta")
        article_id = p.findall(".//article-id")

        
        history_date = p.findall(".//date")
        affiliation = p.findall(".//aff")
        corresp = p.findall(".//corresp")
        contrib = p.findall(".//contrib")
        xref = p.findall(".//xref")
        p.findall(".//corresp")

        journal_m = []
        for elem in journal_meta:
            journal_m.append(etree.tostring(elem).decode())

        articles = []
        pubs = []
        for elem in article_id:
            pubs.append(elem.attrib["pub-id-type"])
            articles.append(elem.text)
            
        further_data = {}
        
        for supp in supplementary_data:
            id = supp.attrib["id"]
            if(id not in further_data):
                further_data[id] = {}
            further_data[id]["Supplementary Material"] = ''.join(supp.itertext())
            if "mimetype" in supp.attrib:
                further_data[id]["mimetype"] = supp.attrib["mimetype"]
    
        for x in xref:
            if(x.attrib["ref-type"] == "supplementary-material"):
                id = x.attrib["rid"]
                if(id not in further_data):
                    further_data[id] = {}
                further_data[id]["Xref"] = ''.join(x.itertext())
                further_data[id]["Xref Context"] = ''.join(x.getparent().itertext())

     
        
        corresponding_authors = {}
        for c in corresp:
            id = c.attrib["id"]
            email = c.findall(".//email")
            em = {}
            for e in email:
                dict_insert_collision_avoidance(0, "Mail", e.text, em)
            corresponding_authors[id] = em

        corresponding_authors["Affiliations"] = []
        for c in contrib:
            if("corresp" in c.attrib):
                name = c.find("name")
                if name:
                    corresponding_authors["Name"] = " ".join(name.itertext())
                ref = c.findall(".//xref")
                for aff in ref:
                    if "ref-type" in aff.attrib and "aff" in aff.attrib["ref-type"]:                        
                        corresponding_authors["Affiliations"].append(aff.attrib["rid"])



        affiliations = {}
        for a in affiliation:
            id = a.attrib["id"]
            if id in corresponding_authors["Affiliations"]:
                countries = a.findall(".//addr-line")
                if(len(countries) > 1):
                    print("Assumption wrong that there's only one address per affiliation")
                    assumptions = False
                for c in countries:
                    if(c.text): 
                        country = c.text.split(", ")[-1]
                    else:
                        country = "Missing in source data"
                xml = etree.tostring(a).decode()
                affiliations[id] = { "Country": country, "Raw": xml}
        if(not affiliations):
             affiliations[id] = { "Country": "Missing", "Raw": "Not found"}
        pub_dict = {}
        for pub, art in zip(pubs, articles):
            dict_insert_collision_avoidance(1, pub, art, pub_dict)

        date_history = []
        date_type_history = []
        for elem in history_date:
            date_type_history.append(elem.attrib["date-type"])
            formated_date = []
            for x in elem.itertext():
                if x.strip():
                    formated_date.append(x)
            date_history.append('/'.join(formated_date))
    
        date_dict = {}
        for d, dt in zip(date, date_type):
            dict_insert_collision_avoidance(1, dt, d, date_dict)

        date_hist_dict = {}
        for d, dt in zip(date_history, date_type_history):
            dict_insert_collision_avoidance(1, dt, d, date_hist_dict)


        

        

        if(titles and meta_value and journal_m and methods and article_title):
            d = { "Source File": f, "Journal Title": titles, "Journal Meta": journal_m, "Data Availability": meta_value, "Materials and Methods": methods}
            
            d["Article Title"] = article_title
            if(pub_dict):
                d["Article IDs"] = pub_dict

            if(date_dict or date_hist_dict):
                d["Dates"] = { "Year": year }
                if(date_dict):
                    d["Dates"]["Publications"] = date_dict
                if(date_hist_dict):
                    d["Dates"]["History"] = date_hist_dict
            if(affiliations):
                d["Affiliations"] = affiliations
            if(corresponding_authors):
                d["Corresponding Authors"] = corresponding_authors
            if(further_data):
                d["Further Data"] = further_data
            if(subjects):
                for sub in subjects:
                    subjects[sub] = list(subjects[sub])
                d["Article Type"] = subjects
            if(other):
                d["Other Meta Groups"] = other
            
            data.append(d)
        else:
            print("Article", f,"is missing required properties." )

    
    return data



