
import json
import os
import csv
import re

import extractor
import analyser
import gazetteer
import urlizer


input_folder = 'input/'
output_folder = 'output/'
all_files = os.listdir(input_folder)
existing_output_files = os.listdir(output_folder)

if "output.jsonl" in existing_output_files:
    with open(output_folder + "output.jsonl", "r") as json_infile:
        for line in json_infile.readlines():
            handled_file = json.loads(line)["Source File"]
            if handled_file in all_files:
                all_files.remove(handled_file)
                print("Skipped", handled_file, " because it was already handled.")


with open(output_folder + "output.jsonl", "a") as json_outfile:
    extracted_data = extractor.extract(input_folder, all_files)
    with open(output_folder + "output.csv", "a") as csv_outfile:
        header = ["Journal","Submission date","Publication data","Publication year","DOI","Type of Article","Article Title","Discipline","Country","Section Text:Generated","Generated","Section Text:Shared","Shared","Code location","DA"]
        csvWriter = csv.DictWriter(csv_outfile, fieldnames=header)
        if "output.csv" not in existing_output_files:
            csvWriter.writeheader()

        relevant = {}
        illegal_mimetypes = ["audio", "drawing", "image" "video", "font"]
        for idx, file in enumerate(extracted_data):
            print("Handling", idx + 1, "of", len(extracted_data), " - ", file["Source File"], flush=True)
            relevant = { 
                "Data Availability": {
                    "Source": file["Data Availability"],
                    "urls": urlizer.urlize(file["Data Availability"]),
                    "SharingTriggers": gazetteer.gazette(file["Data Availability"], gazetteer.GAZETTEER_SHARING),
                    "GeneratingTriggers": gazetteer.gazette(file["Data Availability"], gazetteer.GAZETTEER_GENERATING),
                    "oddpub": analyser.analyse(file["Data Availability"])
                },
                "Materials and Methods": { 
                    "Source": file["Materials and Methods"],
                    "GeneratingTriggers": gazetteer.gazette(file["Materials and Methods"], gazetteer.GAZETTEER_GENERATING),
                    },
                "Supplementary Material": []
            }
            for subsection in file["Further Data"]:
                potential_code_file = True
                if "mimetype" in file["Further Data"][subsection]:
                    for mime in illegal_mimetypes:
                        potential_code_file &= mime not in file["Further Data"][subsection]["mimetype"]
                            


                relevant["Supplementary Material"].append({
                    "Source": file["Further Data"][subsection]["Supplementary Material"],
                    "urls": urlizer.urlize(file["Further Data"][subsection]["Supplementary Material"]),
                    "SharingTriggers": gazetteer.gazette(file["Further Data"][subsection]["Supplementary Material"], gazetteer.GAZETTEER_SHARING_SUPPLEMENTARY),
                    "GeneratingTriggers": gazetteer.gazette(file["Further Data"][subsection]["Supplementary Material"], gazetteer.GAZETTEER_GENERATING),
                    "oddpub": analyser.analyse(file["Further Data"][subsection]["Supplementary Material"]),
                    "Potential Code File": potential_code_file
                })
            
            file["Analysis"] = relevant
            file["Results Share"] = {}
            file["Results"] = {}
            

            def checkSharing(data, section_type):
                sectionTextGazetteer = set()
                sectionTextOddpub = set()
                
                isGazetteer = False
                isOddpub = False

                for oddpub in data["oddpub"]:
                    if oddpub["is_open_code"]:
                        isOddpub = True
                        sectionTextOddpub.add(section_type)

                if data["SharingTriggers"]:
                    isGazetteer = True
                    sectionTextGazetteer.add(section_type)

                isSharing = False
                urls = False
                non_image = False

                if isGazetteer and isOddpub:
                    isSharing = True
                
                if data["urls"] and (isGazetteer or isOddpub):
                    isSharing = True
                    urls = True
    
                if "Supplementary" in section_type and data["Potential Code File"] and (isGazetteer or isOddpub):
                    isSharing = True
                    non_image = True
                
                return isSharing, isGazetteer, isOddpub, urls, non_image, sectionTextGazetteer, sectionTextOddpub
                    



            isSharing, isGazetteer, isOddpub, urls, non_image, sectionTextGazetteer, sectionTextOddpub = checkSharing(relevant["Data Availability"], "DA")
            
            for supp_data in relevant["Supplementary Material"]:
                results = checkSharing(supp_data, "Supplementary")
                isSharing |= results[0]
                isGazetteer |= results[1]
                isOddpub |= results[2]
                urls |= results[3]
                non_image |= results[4]
                sectionTextGazetteer = set.union(sectionTextGazetteer, results[5])
                sectionTextOddpub = set.union(sectionTextOddpub, results[6])

            file["Results Share"]["oddpubb"] = isOddpub
            file["Results Share"]["Gazetteer"] = isGazetteer
            file["Results Share"]["Url Extraction"] = urls
            file["Results Share"]["Non-Images"] = non_image

            sectionText = "N/A"
            codeLocation = "N/A"
            shared = "No"

            if isSharing or isOddpub:
                if not isSharing:
                    sectionText = "|".join(sectionTextOddpub)
                elif not isOddpub:
                    sectionText = "|".join(sectionTextGazetteer)
                else:
                    sectionText = "|".join(set.union(sectionTextGazetteer, sectionTextOddpub))
                    shared = "Yes"

                if urls:
                    codeLocation = "ONLINE"
                    shared = "Yes"
                elif non_image:
                    codeLocation = "Supplementary Information"
                    shared = "Yes"


            file["Results"]["Section Text:Shared"] = sectionText
            file["Results"]["Shared"] = shared
            file["Results"]["Code Location"] = codeLocation

            sectionTextGenerated = []
            if relevant["Data Availability"]["GeneratingTriggers"]:
                sectionTextGenerated.append("DA")
            if relevant["Materials and Methods"]["GeneratingTriggers"]:
                sectionTextGenerated.append("Method")
            supp_data_exists = False
            for supp_data in relevant["Supplementary Material"]:
                supp_data_exists |= any(supp_data["GeneratingTriggers"])
            if supp_data_exists:
                sectionTextGenerated.append("Supplementary")

            if any(sectionTextGenerated):
                file["Results"]["Section Text: Generated"] = "|".join(sectionTextGenerated)
                file["Results"]["Generated"] = "Yes"
            else:
                file["Results"]["Section Text: Generated"] = "N/A"
                file["Results"]["Generated"] = "No"


            discipline = "Missing"
            if "Discipline-v3" in file["Article Type"]:
                discipline = ",".join(file["Article Type"]["Discipline-v3"])

            countries = set()
            for affiliation in file["Affiliations"]:
                countries.add(file["Affiliations"][affiliation]["Country"])
            if(file["Journal Title"][0]):
                csvWriter.writerow({
                        "Journal": file["Journal Title"][0],
                        "Submission date": file["Dates"]["History"]["received"],
                        "Publication data": file["Dates"]["History"]["accepted"],
                        "Publication year": file["Dates"]["Year"],
                        "DOI": file["Article IDs"]["doi"],
                        "Type of Article": file["Article Type"]["heading"][0],
                        "Article Title": file["Article Title"],
                        "Discipline": discipline,
                        "Country": ",".join(countries),
                        "Section Text:Generated": file["Results"]["Section Text: Generated"],
                        "Generated": file["Results"]["Generated"],
                        "Section Text:Shared": file["Results"]["Section Text:Shared"],
                        "Shared": file["Results"]["Shared"],
                        "Code location":  file["Results"]["Code Location"],
                        "DA": re.sub(r"\s(?=\s)", "", re.sub(r"\s", ' ', file["Data Availability"]))
                    })
            else:
                print("something went really wrong with", file["Article IDs"]["doi"])
            json.dump(file, json_outfile)
            json_outfile.write("\n")





out = []
with open(output_folder + "output.jsonl", "r") as json_infile:
    for line in json_infile.readlines():
        out.append(json.loads(line))

with open(output_folder + "output.json", "w") as json_outfile:
    json.dump(out, json_outfile, indent=4)
